@@ -95,7 +95,9 @@ class Meta(type):
                 def f(a, b):
                     self = a
                     # print(op, a, b)
-                    if self.df.is_category(self.expression) and self.df._encode_categoricals and not isinstance(b, Expression):
+                    if isinstance(b, str) and self.dtype.is_datetime:
+                        b = np.datetime64(b)
+                    if self.df.is_category(self.expression) and self.df._future_behaviour and not isinstance(b, Expression):
                         labels = self.df.category_labels(self.expression)
                         if b not in labels:
                             raise ValueError(f'Value {b} not present in {labels}')
@@ -222,6 +224,10 @@ class Expression(with_metaclass(Meta)):
                 return value
         return self.expression
 
+    def fingerprint(self):
+        fp = vaex.cache.fingerprint(self.expression, self.df.fingerprint())
+        return f'expression-{fp}'
+
     def copy(self, df=None):
         """Efficiently copies an expression.
 
@@ -314,6 +320,10 @@ class Expression(with_metaclass(Meta)):
     def shape(self):
         return self.df._shape_of(self)
 
+    @property
+    def ndim(self):
+        return 1 if self.dtype.is_list else len(self.df._shape_of(self))
+
     def to_arrow(self, convert_to_native=False):
         '''Convert to Apache Arrow array (will byteswap/copy if convert_to_native=True).'''
         values = self.values
@@ -580,16 +590,30 @@ class Expression(with_metaclass(Meta)):
         '''
         expression = self
         if axis is None:
-            axis = [0]
             dtype = self.dtype
-            while dtype.is_list:
-                axis.append(axis[-1] + 1)
-                dtype = dtype.value_type
+            if dtype.is_list:
+                axis = [0]
+                while dtype.is_list:
+                    axis.append(axis[-1] + 1)
+                    dtype = dtype.value_type
+            elif self.ndim > 1:
+                axis = list(range(self.ndim))
+            else:
+                axis = [0]
         elif not isinstance(axis, list):
             axis = [axis]
             axis = list(set(axis))  # remove repeated elements
         dtype = self.dtype
-        if 1 in axis:
+        if self.ndim > 1:
+            array_axes = axis.copy()
+            if 0 in array_axes:
+                array_axes.remove(0)
+            expression = expression.array_sum(axis=array_axes)
+            for i in array_axes:
+                axis.remove(i)
+                del i
+            del array_axes
+        elif 1 in axis:
             if self.dtype.is_list:
                 expression = expression.list_sum()
                 if axis:
@@ -598,6 +622,8 @@ class Expression(with_metaclass(Meta)):
                 raise ValueError(f'axis=1 not supported for dtype={dtype}')
         if axis and axis[0] != 0:
             raise ValueError(f'Only axis 0 or 1 is supported')
+        if expression.ndim > 1:
+            raise ValueError(f'Cannot sum non-scalar (ndim={expression.ndim})')
         if axis is None or 0 in axis:
             kwargs = dict(locals())
             del kwargs['self']
@@ -895,7 +921,7 @@ def f({0}):
             return self.df['isin_set(%s, %s)' % (self, var)]
         else:
             if self.is_string():
-                values = vaex.column._to_string_sequence(values)
+                values = pa.array(values)
             else:
                 values = np.array(values, dtype=self.dtype.numpy)
             var = self.df.add_variable('isin_values', values, unique=True)
@@ -1119,6 +1145,9 @@ class FunctionSerializablePickle(FunctionSerializable):
         self.f = f
         self.multiprocessing = multiprocessing
 
+    def __eq__(self, rhs):
+        return self.f == rhs.f
+
     def pickle(self, function):
         return pickle.dumps(function)
 
@@ -1316,7 +1345,6 @@ class Function(object):
         expression = "{}({})".format(self.name, arg_string)
         return Expression(self.dataset, expression)
 
-
 class FunctionBuiltin(object):
 
     def __init__(self, dataset, name, **kwargs):
